#include "wrap/thread.h"
#include "wrap/synch.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define TCOUNT 16
#define TITER 100000
#define SIZE 64
#define AMOUNT 1000

struct account {
	int balance;
	char name[32];
	struct lock lock;
};

struct account account[SIZE];

static bool transfer(int amount, int from, int to) {
	bool ok = false;
	lock_acquire(&account[from].lock);
	lock_acquire(&account[ to ].lock);

	if (account[from].balance >= amount) {
		account[from].balance -= amount;
		account[ to ].balance += amount;
		ok = true;
	}

	lock_release(&account[from].lock);
	lock_release(&account[ to ].lock);

	if (!ok) {
		// fprintf(stderr, "ERROR: %s saknar täckning\n", account[from].name);
	}

	return ok;
}

static void random_transfer(void *input) {
	struct semaphore *sema = input;
	int i;
	for (i = 0; i < TITER; ++i) {
		int from   = rand() % SIZE;
		int to     = rand() % SIZE;
		int amount = rand() % AMOUNT;
		transfer(amount, from, to);
	}

	sema_up(sema);
}

static void print_total_balance(void) {
	int total_balance = 0;
	int i;
	for (i = 0; i < SIZE; ++i) {
		total_balance += account[i].balance;
	}
	printf("Total balance: %d\n", total_balance);
}

static void print_negative_balance(void) {
	int i;
	for (i = 0; i < SIZE; ++i) {
		if (account[i].balance < 0) {
			fprintf(stderr, "Negative balance on account %s\n", account[i].name);
		}
	}
}

static void init_accounts(int initial_balance) {
	int i;
	for (i = 0; i < SIZE; ++i) {
		account[i].balance = initial_balance;
		snprintf(account[i].name, 32, "Account %d", i);
		lock_init(&account[i].lock);
	}
}

int main(void) {
	struct semaphore t[TCOUNT];
	int i;

	srand((int)time(NULL));

	init_accounts(AMOUNT);
	print_total_balance();
	print_negative_balance();

	for (i = 0; i < TCOUNT; ++i) {
		sema_init(&t[i], 0);
		thread_create("", 0, &random_transfer, &t[i]);
	}

	for (i = 0; i < TCOUNT; ++i) {
		sema_down(&t[i]);
		sema_destroy(&t[i]);
	}

	print_total_balance();
	print_negative_balance();

	return 0;
}
