#include "main.h"
#include "wrap/atomics.h"
#include <stdlib.h>

struct my_lock {
	int busy;
	int owner;
};

struct my_lock *my_lock_create(void) {
	struct my_lock *l = malloc(sizeof(struct my_lock));

	l->busy = 0;
	l->owner = -1;

	return l;
}

void my_lock_destroy(struct my_lock *l) {
	free(l);
}

void my_lock_acquire(struct my_lock *l, int tid) {
	/* Option 1: test_and_set */
	/* while (test_and_set(&l->busy) == 1) */
	/* 	; */

	/* Option 2: atomic_swap */
	/* while (atomic_swap(&l->busy, 1) == 1) */
	/* 	; */

	/* Option 3: compare_and_swap */
	while (compare_and_swap(&l->busy, 0, 1) == 1)
		;

	l->owner = tid;
}

void my_lock_release(struct my_lock *l, int) {
	l->owner = -1;
	atomic_write(&l->busy, 0);
}
