/**
 * This is the main function of the print program.
 *
 * Compile it with different lock implementations to observe different
 * behaviors. For example:
 *
 * - gcc main.c busyflag.c -o busyflag
 */


#include "wrap/thread.h"
#include "wrap/synch.h"
#include "main.h"
#include <stdio.h>
#include <stdlib.h>

// Large enough to see errors.
const int times_to_print = 500000;

// Lock for stdout.
struct my_lock *stdout_lock;

// Sequential number for printing.
int seq;

// Semaphore to indicate when threads are done.
struct semaphore done_sema;

static void thread_a(void *) {
	for (int i = 0; i < times_to_print; i++) {
		my_lock_acquire(stdout_lock, 0);
		fprintf(stdout, "%04d: Thread A\n", ++seq);
		my_lock_release(stdout_lock, 0);
	}

	sema_up(&done_sema);
}

static void thread_b(void *) {
	for (int i = 0; i < times_to_print; i++) {
		my_lock_acquire(stdout_lock, 1);
		fprintf(stdout, "%04d: Thread B\n", ++seq);
		my_lock_release(stdout_lock, 1);
	}

	sema_up(&done_sema);
}

int main(void) {
	sema_init(&done_sema, 0);
	stdout_lock = my_lock_create();

	thread_new(&thread_a, NULL);
	thread_new(&thread_b, NULL);

	sema_down(&done_sema);
	sema_down(&done_sema);

	my_lock_destroy(stdout_lock);

	return 0;
}
