#include "main.h"
#include <stdlib.h>
#include <stdbool.h>

struct my_lock {
	bool i_want_in[2];
	int whose_turn;
};

struct my_lock *my_lock_create(void) {
	struct my_lock *l = malloc(sizeof(struct my_lock));

	l->i_want_in[0] = false;
	l->i_want_in[1] = false;
	l->whose_turn = 0;

	return l;
}

void my_lock_destroy(struct my_lock *l) {
	free(l);
}

#define ME(tid) (tid)
#define OTHER(tid) (!tid)

/* Peterson's solution.
 *
 * Presumption: Only two threads.
 *
 * Switch at note 1 or 2:
 *  First to write whose_turn will win the lock
 *  Last to write whose_turn will wait
 *
 * Switch at note 3:
 *  If turn becomes mine other will wait until I do not want in anymore
 *  If other want in he'll set turn to mine and have to wait
 *
 * If I'm the only one that want in I'll get in
 * If I wait I'll continue when other get out
 */
void my_lock_acquire(struct my_lock *l, int tid) {
	l->i_want_in[ME(tid)] = true;
	/* note 1 */
	l->whose_turn = OTHER(tid);
	/* note 2 */
	while ( l->i_want_in[OTHER(tid)] && /* note 3 */
		l->whose_turn == OTHER(tid) )
		; /* busy wait */
}

void my_lock_release(struct my_lock *l, int tid) {
	l->i_want_in[ME(tid)] = false;
}
